function [xc, yc, cpa] = vortex_panel_method (x, y, aagrad)

% -------------------------------------------------------------------------
% INPUT
% -------------------------------------------------------------------------

x = x';
y = y';

aarad = aagrad*pi/180;
ncoord = length(x);
npanel = ncoord - 1;

% -------------------------------------------------------------------------
% CORE
% -------------------------------------------------------------------------

for i = 1:npanel
    xc(i) = (x(i) + x(i+1))/2;
    yc(i) = (y(i) + y(i+1))/2;
    tpanel(i) = ((x(i+1) - x(i))^2 + (y(i+1) - y(i))^2)^(0.5);
    apanel(i) = atan2((y(i+1) - y(i)) , (x(i+1) - x(i)));
    rhs(i) = sin(apanel(i) - aarad);
end

% A continuacin se calculan los coeficientes usados en la Matriz Anij de
% la ecuacin 5.47. Los nombres de las variables se corresponden con los
% usados en el libro (pgs. 157 - 159) y por eso no se especifican al
% comienzo del cdigo en el apartado de variables.

% Falta inicializar cn1, cn2, ct1, ct2
for i = 1:npanel
    for j = 1:npanel
        if i == j
            cn1(i,j) = -1;
            cn2(i,j) = 1;
            ct1(i,j) = pi/2;
            ct2(i,j) = pi/2;
        else
            A = -(xc(i)-x(j))*cos(apanel(j))-(yc(i)-y(j))*sin(apanel(j));
            B = (xc(i)-x(j))^2+(yc(i)-y(j))^2;
            C = sin(apanel(i)-apanel(j));
            D = cos(apanel(i)-apanel(j));
            E = (xc(i)-x(j))*sin(apanel(j))-(yc(i)-y(j))*cos(apanel(j));
            F = log(1+(tpanel(j)^2+2*A*tpanel(j))/B);
            G = atan2((E*tpanel(j)),(B+A*tpanel(j)));
            P = (xc(i)-x(j))*sin(apanel(i)-2*apanel(j))+ ...
                (yc(i)-y(j))*cos(apanel(i)-2*apanel(j));
            Q = (xc(i)-x(j))*cos(apanel(i)-2*apanel(j))- ...
                (yc(i)-y(j))*sin(apanel(i)-2*apanel(j));
            cn2(i,j) = D+0.5*Q*F/tpanel(j)-(A*C+D*E)*G/tpanel(j);
            cn1(i,j) = 0.5*D*F+C*G-cn2(i,j);
            ct2(i,j) = C+0.5*P*F/tpanel(j)+(A*D-C*E)*G/tpanel(j);
            ct1(i,j) = 0.5*C*F-D*G-ct2(i,j);
        end
    end
end

% Se calculan los coeficientes usados en las Eqs 5.47 (Anij) y 5.49 (Atij)

% Falta inicializar an, at.
for i = 1:npanel
    an(i,1) = cn1(i,1);
    an(i,ncoord) = cn2(i,npanel);
    at(i,1) = ct1(i,1);
    at(i,ncoord) = ct2(i,npanel);
    for j = 2:npanel
        an(i,j) = cn1(i,j)+cn2(i,(j-1));
        at(i,j) = ct1(i,j)+ct2(i,(j-1));
    end
end

% Se establece la condicin de Kutta.
an(ncoord,1) = 1;
an(ncoord,ncoord) = 1;
rhs(ncoord) = 0;

for j = 2:npanel
    an(ncoord,j) = 0;
end

% Se hace la transformada de rhs.
rhs = rhs';

% Se calculan los 'vortex strengths' resolviendo el sistema de ecuaciones 
% lineales descritas por la Eq. 5.47.
g = an\rhs;

% Para finalizar el anlisis del fluido, se calculan las velocidades 
% paralelas a los paneles (absoluta y 'dimmensionless') y los coeficientes
% de presiones en los paneles.  
cl = 0;

for i = 1:npanel
    smt = 0;
    for j = 1:ncoord
        sm = at(i,j)*g(j);
        smt = smt+sm;
    end
    
    % Falta inicializar vd y cpa.
    
    % En vd se almacenan las velocidades paralelas a los paneles en los 
    % puntos de control (ver Eqs. 5.48 y 5.49, y el prrafo encima de la 
    % Eq. 5.48).
    vd(i) = (cos(apanel(i)-aarad)+smt);
    
    % En cpa se almacenan los coeficientes de presiones en los puntos de 
    % control.
    cpa(i) = 1-(vd(i))^2;
    cl = cl-cpa(i)*tpanel(i)*cos(apanel(i)-aarad);
end
